# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from collections.abc import Iterable

import pytest

from odfdo.document import Document
from odfdo.header import Header
from odfdo.line_break import LineBreak
from odfdo.link import Link
from odfdo.list import List
from odfdo.paragraph import Paragraph
from odfdo.spacer import Spacer
from odfdo.tab import Tab
from odfdo.utils.remove_tree import remove_tree


@pytest.fixture
def document(samples) -> Iterable[Document]:
    document = Document("odt")
    document.body.clear()
    document.body.append(Header(1, "header"))
    para = Paragraph("first\tsecond\nthird    four")
    link = Link("http://example.com/", name="link1")
    para.append(link)
    document.body.append(para)
    yield document


def test_remove_no_found(document):
    remove_tree(document.body, List)
    expected = (
        "<office:text>"
        '<text:h text:outline-level="1">header</text:h>'
        "<text:p>"
        "first<text:tab/>"
        "second<text:line-break/>"
        'third <text:s text:c="3"/>'
        "four"
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() == expected


def test_remove_link(document):
    remove_tree(document.body, Link)
    expected = (
        "<office:text>"
        '<text:h text:outline-level="1">header</text:h>'
        "<text:p>"
        "first<text:tab/>"
        "second<text:line-break/>"
        'third <text:s text:c="3"/>'
        "four"
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() == expected


def test_remove_tab(document):
    remove_tree(document.body, Tab)
    expected = (
        "<office:text>"
        '<text:h text:outline-level="1">header</text:h>'
        "<text:p>"
        "first second<text:line-break/>"
        'third <text:s text:c="3"/>four'
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() == expected


def test_remove_spacer(document):
    remove_tree(document.body, Spacer)
    expected = (
        "<office:text>"
        '<text:h text:outline-level="1">header</text:h>'
        "<text:p>"
        "first<text:tab/>"
        "second<text:line-break/>"
        "third "
        "four"
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() == expected


def test_remove_line_break(document):
    remove_tree(document.body, LineBreak)
    expected1 = (
        "<office:text>"
        '<text:h text:outline-level="1">header</text:h>'
        "<text:p>"
        "first<text:tab/>"
        "second "
        'third <text:s text:c="3"/>'
        "four"
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    expected2 = (
        "<office:text>"
        '<text:h text:outline-level="1">header</text:h>'
        "<text:p>"
        "first<text:tab/>"
        "second "
        'third<text:s text:c="4"/>'
        "four"
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() in (expected1, expected2)


def test_remove_header_wrong(document):
    # when removing header, we want to also remove the inner text
    remove_tree(document.body, Header)
    expected = (
        "<office:text>"
        "header"  # bad
        "<text:p>"
        "first"
        "<text:tab/>"
        "second<text:line-break/>"
        'third <text:s text:c="3"/>'
        "four"
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() == expected


def test_remove_header(document):
    # when removing header, we want to also remove the inner text
    remove_tree(document.body, Header, keep_children=False)
    expected = (
        "<office:text>"
        "<text:p>"
        "first"
        "<text:tab/>"
        "second<text:line-break/>"
        'third <text:s text:c="3"/>'
        "four"
        '<text:a xlink:href="http://example.com/" office:name="link1"/>'
        "</text:p>"
        "</office:text>"
    )
    assert document.body.serialize() == expected



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Style utilities function."""

from __future__ import annotations

import contextlib
from copy import deepcopy
from typing import Any

from .const import ODF_PROPERTIES
from .element import Element
from .style_base import PropDict
from .utils.style_constants import _BASE_PROPERTY_MAPPING, STYLE_ATTRIBUTES


def _merge_dicts(dic_base: dict, *args: dict, **kwargs: Any) -> dict:
    """Merge two or more dictionaries into a new dictionary object.

    Args:
        dic_base: The base dictionary.
        *args: Additional dictionaries to merge.
        **kwargs: Keyword arguments to merge.

    Returns:
        dict: A new dictionary containing the merged content.
    """
    new_dict = deepcopy(dic_base)
    for dic in args:
        new_dict.update(dic)
    new_dict.update(kwargs)
    return new_dict


def _generate_property_mapping() -> dict[str, str]:
    # add <style:paragraph-properties> <style:text-properties>
    all_attributes = (
        STYLE_ATTRIBUTES["paragraph"]
        | STYLE_ATTRIBUTES["text"]
        | STYLE_ATTRIBUTES["page-layout"]
        | STYLE_ATTRIBUTES["ruby"]
        | STYLE_ATTRIBUTES["section"]
        | STYLE_ATTRIBUTES["table"]
        | STYLE_ATTRIBUTES["table-column"]
        | STYLE_ATTRIBUTES["table-row"]
        | STYLE_ATTRIBUTES["table-cell"]
        | STYLE_ATTRIBUTES["graphic"]
    )
    map_1 = {
        a[6:].replace("-", "_"): a for a in all_attributes if a.startswith("style:")
    }
    map_2 = {
        a[5:].replace("-", "_"): a for a in all_attributes if a.startswith("text:")
    }
    map_3 = {
        a.replace("-", "_").replace(":", "_"): a
        for a in all_attributes
        if a.startswith("dr3d:")
    }
    map_4 = {
        a.replace("-", "_").replace(":", "_"): a
        for a in all_attributes
        if a.startswith("draw:")
    }
    map_5 = {
        a.replace("-", "_").replace(":", "_"): a
        for a in all_attributes
        if a.startswith("svg:")
    }
    map_6 = {
        a.replace("-", "_").replace(":", "_"): a
        for a in all_attributes
        if a.startswith("table:")
    }
    return _merge_dicts(
        _BASE_PROPERTY_MAPPING, map_1, map_2, map_3, map_4, map_5, map_6
    )


_PROPERTY_MAPPING = _generate_property_mapping()


def _map_key(key: str) -> str | None:
    if key in ODF_PROPERTIES:
        return key
    key = _PROPERTY_MAPPING.get(key, key).replace("_", "-")
    if ":" not in key:
        key = f"fo:{key}"
    if key in ODF_PROPERTIES:
        return key
    return None


def _expand_properties_dict(properties: PropDict) -> PropDict:
    """Expand a dictionary of properties by mapping keys to their full ODF attribute names.

    Args:
        properties: A dictionary of properties with potentially simplified keys.

    Returns:
        dict[str, str | dict]: A new dictionary with keys mapped to full ODF attribute names.
    """
    expanded = {}
    for key in sorted(properties.keys()):
        prop_key = _map_key(key)
        if prop_key and key != prop_key:
            expanded[prop_key] = properties[key]
            continue
        if key not in expanded:
            expanded[key] = properties[key]
    return expanded


def _expand_properties_list(properties: list[str]) -> list[str]:
    """Expand a list of property keys by mapping them to their full ODF attribute names.

    Args:
        properties: A list of property keys with potentially simplified names.

    Returns:
        list[str]: A new list with keys mapped to full ODF attribute names.
    """
    return list(filter(None, (_map_key(key) for key in properties)))


def _check_background_support(family: str) -> None:
    """Check if the given style family supports background properties.

    Args:
        family: The style family to check.

    Raises:
        TypeError: If the family does not support background properties.
    """
    if family not in {
        "text",
        "paragraph",
        "page-layout",
        "section",
        "table",
        "table-row",
        "table-cell",
        "graphic",
    }:
        raise TypeError(f"No background support for family {family!r}")


def _check_position(position: str | None) -> None:
    """Validate a background position string.

    Args:
        position: The background position string.

    Raises:
        ValueError: If the position string is not well-formatted or contains
            unknown keywords.
    """
    if not position:
        return
    parts = position.split()
    if not parts:
        raise ValueError("Wrong formatted background position attribute")
    for word in parts:
        if word not in {"left", "center", "right", "top", "bottom"}:
            raise ValueError(f"Unknown background position {position!r}")


def _check_repeat(repeat: str | None) -> None:
    """Validate a background repeat string.

    Args:
        repeat: The background repeat string.

    Raises:
        ValueError: If the repeat string is not well-formatted or contains
            unknown keywords.
    """
    if not repeat:
        return
    parts = repeat.split()
    if not parts:
        raise ValueError("Incorrect background repeat attribute")
    for word in parts:
        if word not in {"no-repeat", "repeat", "stretch"}:
            raise ValueError(f"Unknown background repeat {repeat!r}")


def _check_opacity(opacity: str | int | None) -> None:
    """Validate an opacity value.

    Args:
        opacity: The opacity value (0-100).

    Raises:
        ValueError: If the opacity value is outside the valid range (0-100).
    """
    if not opacity:
        return
    value = int(opacity)
    if value < 0 or value > 100:
        raise ValueError(f"Incorrect opacity {opacity!r}")


def _erase_background(element: Element) -> None:
    """Erase background properties (color and image) from the given element.

    Args:
        element: The element from which to erase background properties.
    """
    family = element.family  # type: ignore[attr-defined]
    properties = element.get_element(f"style:{family}-properties")
    if properties is None:
        return
    with contextlib.suppress(KeyError):
        properties.del_attribute("fo:background-color")
    bg_image = properties.get_element("style:background-image")
    if bg_image is not None:
        properties.delete(bg_image)


def _set_background_color(element: Element, color: str) -> None:
    """Set the background color for the given element.

    If a background image exists, it will be removed.

    Args:
        element: The element to set the background color for.
        color: The color string (e.g., "#RRGGBB" or "red").
    """
    family = element.family  # type: ignore[attr-defined]
    properties = element.get_element(f"style:{family}-properties")
    if properties is None:
        properties = Element.from_tag(f"style:{family}-properties")
        element.append(properties)
    properties.set_attribute("fo:background-color", color)
    bg_image = properties.get_element("style:background-image")
    if bg_image is not None:
        properties.delete(bg_image)


def _set_background_image(
    element: Element,
    url: str | None,
    position: str | None,
    repeat: str | None,
    opacity: str | int | None,
    filter: str | None,  # noqa: A002
) -> None:
    """Set the background image for the given element.

    If a background color exists, it will be removed.

    Args:
        element: The element to set the background image for.
        url: The URL of the image.
        position: The position of the background image.
        repeat: How the background image is repeated.
        opacity: The opacity of the background image (0-100).
        filter: A filter to apply to the image.
    """
    _check_position(position)
    _check_repeat(repeat)
    _check_opacity(opacity)
    family = element.family  # type: ignore[attr-defined]
    properties = element.get_element(f"style:{family}-properties")
    if properties is None:
        properties = Element.from_tag(f"style:{family}-properties")
        element.append(properties)
    # properties.set_attribute("fo:background-color", "transparent")
    with contextlib.suppress(KeyError):
        properties.del_attribute("fo:background-color")
    bg_image = properties.get_element("style:background-image")
    if bg_image is None:
        bg_image = Element.from_tag("style:background-image")
        properties.append(bg_image)
    bg_image.url = url  # type:ignore
    if position:
        bg_image.position = position  # type:ignore
    if repeat:
        bg_image.repeat = repeat  # type:ignore
    if opacity:
        bg_image.opacity = str(opacity)  # type:ignore
    if filter:
        bg_image.filter = filter  # type:ignore


def _set_background(
    element: Element,
    color: str | None,
    url: str | None,
    position: str | None,
    repeat: str | None,
    opacity: str | int | None,
    filter: str | None,  # noqa: A002
) -> None:
    """Set the background properties (color or image) for an element.

    This function handles setting either a background color or a background image,
    depending on the provided arguments. It validates background-related properties
    and ensures that conflicting properties (e.g., both color and image) are
    handled correctly.

    Args:
        element: The element to set the background for.
        color: The background color string.
        url: The URL of the background image.
        position: The position of the background image.
        repeat: How the background image is repeated.
        opacity: The opacity of the background (0-100).
        filter: A filter to apply to the background image.

    Raises:
        TypeError: If a background image is specified for a text style.
    """
    family = element.family  # type: ignore[attr-defined]
    _check_background_support(family)
    if url is not None and family == "text":
        raise TypeError("No background image for text styles")
    if color:
        return _set_background_color(element, color)
    if url:
        return _set_background_image(element, url, position, repeat, opacity, filter)
    return _erase_background(element)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
from datetime import timedelta

from odfdo.datatype import Duration
from odfdo.meta_auto_reload import MetaAutoReload


def test_create_missing_arg():
    reload = MetaAutoReload()
    assert reload.delay == Duration.encode(timedelta(0))


def test_create_minimal():
    delay = timedelta(seconds=15)
    reload = MetaAutoReload(delay)
    expected = (
        '<meta:auto-reload xlink:actuate="onLoad" xlink:show="replace" '
        'xlink:type="simple" meta:delay="PT00H00M15S" xlink:href=""/>'
    )
    assert reload.serialize() == expected
    assert reload.href == ""


def test_create():
    delay = timedelta(seconds=15)
    reload = MetaAutoReload(delay=delay, href="some url")
    expected = (
        '<meta:auto-reload xlink:actuate="onLoad" xlink:show="replace" '
        'xlink:type="simple" meta:delay="PT00H00M15S" xlink:href="some url"/>'
    )
    assert reload.serialize() == expected
    assert reload.delay == Duration.encode(delay)
    assert reload.href == "some url"
    assert repr(reload) == (
        "<MetaAutoReload tag=meta:auto-reload href=some url delay=0:00:15>"
    )


def test_as_dict():
    delay = timedelta(seconds=15)
    reload = MetaAutoReload(delay=delay, href="some url")
    expected = {
        "meta:delay": delay,
        "xlink:actuate": "onLoad",
        "xlink:href": "some url",
        "xlink:show": "replace",
        "xlink:type": "simple",
    }
    print(reload.as_dict())
    assert reload.as_dict() == expected


def test_as_dict_delay_none():
    reload = MetaAutoReload(href="some url")
    reload.delay = None
    expected = {
        "meta:delay": timedelta(0),
        "xlink:actuate": "onLoad",
        "xlink:href": "some url",
        "xlink:show": "replace",
        "xlink:type": "simple",
    }
    print(reload.as_dict())
    assert reload.as_dict() == expected


def test_from_dict():
    delay = timedelta(seconds=15)
    reload = MetaAutoReload()
    imported = {
        "meta:delay": delay,
        "xlink:actuate": "onLoad",
        "xlink:href": "some url",
        "xlink:show": "replace",
        "xlink:type": "simple",
    }
    reload.from_dict(imported)
    assert reload.as_dict() == imported


def test_str():
    delay = timedelta(seconds=15)
    reload = MetaAutoReload(delay=delay, href="some url")
    expected = "(some url)"
    assert str(reload) == expected


def test_repr():
    delay = timedelta(seconds=15)
    reload = MetaAutoReload(delay=delay, href="some url")
    expected = "<MetaAutoReload tag=meta:auto-reload href=some url delay=0:00:15>"
    assert repr(reload) == expected


def test_repr_2():
    reload = MetaAutoReload()
    reload.delay = None
    expected = "<MetaAutoReload tag=meta:auto-reload href= delay=>"
    assert repr(reload) == expected



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          Romain Gauthier <romain@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>
"""Note class for "text:note" tag."""

from __future__ import annotations

from typing import Any, ClassVar, Union, cast

from .annotation import Annotation, AnnotationEnd, get_unique_office_name  # noqa: F401
from .element import Element, PropDef, PropDefBool, register_element_class
from .mixin_link import LinkMixin
from .mixin_list import ListMixin
from .mixin_md import MDNote
from .mixin_toc import TocMixin
from .section import SectionMixin


class NoteMixin(Element):
    """Mixin class for classes containing Notes.

    Used by the following classes: "text:a", "text:h", "text:meta", "text:meta-field",
    "text:p", "text:ruby-base", "text:span". And with "office:text" for compatibility
    with previous versions.
    """

    def get_notes(
        self,
        note_class: str | None = None,
        content: str | None = None,
    ) -> list[Note]:
        """Retrieve all notes (`text:note`) that match the specified criteria.

        Args:
            note_class: Filter by note class ("footnote" or "endnote").
            content: A regular expression to match against the note's content.

        Returns:
            list[Note]: A list of `Note` instances matching the criteria.
        """
        return cast(
            list[Note],
            self._filtered_elements(
                "descendant::text:note", note_class=note_class, content=content
            ),
        )

    def get_note(
        self,
        position: int = 0,
        note_id: str | None = None,
        note_class: str | None = None,
        content: str | None = None,
    ) -> Note | None:
        """Retrieve a specific note (`text:note`) that matches the specified criteria.

        Args:
            position: The 0-based index of the matching note to retrieve if multiple match
                the other criteria. Defaults to 0.
            note_id: The unique ID of the note.
            note_class: Filter by note class ("footnote" or "endnote").
            content: A regular expression to match against the note's content.

        Returns:
            Note | None: A `Note` instance matching the criteria, or `None` if not found.
        """
        return cast(
            Union[None, Note],
            self._filtered_element(
                "descendant::text:note",
                position,
                text_id=note_id,
                note_class=note_class,
                content=content,
            ),
        )


class NoteBody(ListMixin, TocMixin, LinkMixin, SectionMixin):
    """Container for the content of a note, "text:note-body"."""

    _tag: str = "text:note-body"
    _properties: tuple[PropDef | PropDefBool, ...] = ()


class Note(MDNote, LinkMixin, Element):
    """A note (footnote or endnote), "text:note".

    Either a footnote or a endnote element with the given text, optionally
    referencing it using the given note_id.
    """

    _tag = "text:note"
    _properties: tuple[PropDef | PropDefBool, ...] = (
        PropDef("note_class", "text:note-class"),
        PropDef("note_id", "text:id"),
    )
    NOTE_CLASS: ClassVar = {"footnote", "endnote"}

    def __init__(
        self,
        note_class: str = "footnote",
        note_id: str | None = None,
        citation: str | None = None,
        body: str | None = None,
        **kwargs: Any,
    ) -> None:
        """Initialize a Note element (footnote or endnote).

        Args:
            note_class: The class of the note ("footnote" or "endnote").
                Defaults to "footnote".
            note_id: A unique ID for the note. If None, one is generated.
            citation: The citation text for the note.
            body: The content of the note body. Can be a string or an `Element`.
            **kwargs: Additional keyword arguments for the parent `Element` class.
        """
        super().__init__(**kwargs)
        if self._do_init:
            self.insert(Element.from_tag("text:note-body"), position=0)
            self.insert(Element.from_tag("text:note-citation"), position=0)
            self.note_class = note_class
            if note_id is not None:
                self.note_id = note_id
            if citation is not None:
                self.citation = citation
            if body is not None:
                self.note_body = body

    @property
    def citation(self) -> str:
        """Get the text content of the note citation.

        Returns:
            str: The citation text, or an empty string if not found.
        """
        note_citation = self.get_element("text:note-citation")
        if note_citation:
            return note_citation.text
        return ""

    @citation.setter
    def citation(self, text: str | None) -> None:
        """Set the text content of the note citation.

        Args:
            text: The new citation text.
        """
        note_citation = self.get_element("text:note-citation")
        if note_citation:
            note_citation.text = text

    @property
    def note_body(self) -> str:
        """Get the text content of the note body.

        Returns:
            str: The content of the note body, or an empty string if not found.
        """
        note_body = self.get_element("text:note-body")
        if note_body:
            return note_body.text_content
        return ""

    @note_body.setter
    def note_body(self, text_or_element: Element | str | None) -> None:
        """Set the content of the note body.

        Args:
            text_or_element: The new content for the note body.
                Can be a string, an `Element`, or None to clear the content.
        """
        note_body = self.get_element("text:note-body")
        if not note_body:
            return None
        if text_or_element is None:
            note_body.text_content = ""
        elif isinstance(text_or_element, str):
            note_body.text_content = text_or_element
        elif isinstance(text_or_element, Element):
            note_body.clear()
            note_body.append(text_or_element)
        else:
            raise TypeError(f'Unexpected type for body: "{type(text_or_element)}"')

    def check_validity(self) -> None:
        """Check the validity of the note's properties.

        Ensures that `note_class` is valid, and that `note_id` and `citation`
        are not empty.

        Raises:
            ValueError: If `note_class` is invalid, or if `note_id` or `citation` are empty.
        """
        if not self.note_class or self.note_class not in self.NOTE_CLASS:
            raise ValueError('Note class must be "footnote" or "endnote"')
        if not self.note_id:
            raise ValueError("Note must have an id")
        if not self.citation:
            raise ValueError("Note must have a citation")
        if not self.note_body:
            pass

    def __str__(self) -> str:
        if self.citation:
            return f"{self.citation}. {self.note_body}"
        return self.note_body


Note._define_attribut_property()

register_element_class(Note)
register_element_class(NoteBody)



#!/usr/bin/env python
"""Export tables to CSV format."""

import os
import sys
from pathlib import Path

from odfdo import Document

_DOC_SEQUENCE = 610
DATA = Path(__file__).parent / "data"
SOURCE = "two_sheets.ods"
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "csv"


def read_source_document() -> Document:
    """Return the source Document."""
    try:
        source = sys.argv[1]
    except IndexError:
        source = DATA / SOURCE
    return Document(source)


def export_tables_to_csv(document: Document) -> None:
    """Export tables to CSV format."""
    for index, table in enumerate(document.body.tables):
        # default parameters produce an "excell" CSV format,
        # see Python csv library for options.
        # Note: body.sheets is an alias for body.tables
        #
        OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
        path = OUTPUT_DIR / f"content_{index}.csv"
        table.to_csv(path)


def main() -> None:
    document = read_source_document()
    export_tables_to_csv(document)
    test_unit(document)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    content0 = document.body.tables[0].to_csv()
    expected0 = "col A,col B,col C\r\n1,2,3\r\na text,,another\r\n"
    assert content0 == expected0

    content1 = document.body.tables[1].to_csv()
    expected1 = ",,,\r\n,col B,col C,col D\r\n,1,2,3\r\n,a text,,another\r\n"
    assert content1 == expected1


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from collections.abc import Iterable
from datetime import datetime, time, timedelta

import pytest

from odfdo.document import Document
from odfdo.element import Element
from odfdo.variable import (
    VarChapter,
    VarCreationDate,
    VarCreationTime,
    VarDate,
    VarDescription,
    VarFileName,
    VarGet,
    VarInitialCreator,
    VarKeywords,
    VarPageCount,
    VarPageNumber,
    VarSet,
    VarSubject,
    VarTime,
    VarTitle,
)

ZOE = "你好 Zoé"


@pytest.fixture
def document(samples) -> Iterable[Document]:
    document = Document(samples("variable.odt"))
    yield document


def test_var_set_class():
    variable = VarSet()
    assert isinstance(variable, VarSet)


def test_var_set_arg_style():
    variable = VarSet(name="myvar", value=42, style="some_bold")
    assert variable.style == "some_bold"


def test_var_set_style_keep():
    variable = VarSet(name="myvar", value=42, style="some_bold")
    variable.set_value(55)
    assert variable.style == "some_bold"


def test_var_set_display_deleted():
    variable = VarSet(name="myvar", value=42, style="some_bold")
    variable.display = None
    variable.set_value(55)
    assert variable.display is None


def test_var_set_display_keep_true():
    variable = VarSet(name="myvar", value=42, style="some_bold")
    variable.display = True
    variable.set_value(55)
    assert variable.display is True


def test_var_set_display_keep_false():
    variable = VarSet(name="myvar", value=42, style="some_bold")
    variable.display = False
    variable.set_value(55)
    assert variable.display is False


def test_var_set_display_no_text():
    variable = VarSet(name="myvar", value=42, style="some_bold")
    variable.display = "none"
    variable.set_value(None)
    assert variable.text == ""


def test_var_get_class():
    variable = VarGet()
    assert isinstance(variable, VarGet)


def test_var_get_arg_style():
    variable = VarGet(name="myvar", style="some_bold")
    assert variable.style == "some_bold"


def test_var_get_arg_from_tag():
    variable = Element.from_tag(
        '<text:variable-get text:name="myvar" style:data-style-name="some_bold">'
        "</text:variable-get>"
    )
    assert isinstance(variable, VarGet)


def test_var_get_arg_from_tag2():
    variable = Element.from_tag(
        '<text:variable-get text:name="myvar" style:data-style-name="some_bold">'
        "</text:variable-get>"
    )
    assert variable.style == "some_bold"
    assert variable.name == "myvar"
    assert not variable.text


def test_var_page_number_class():
    variable = VarPageNumber()
    assert isinstance(variable, VarPageNumber)


def test_var_page_number_from_tag():
    variable = Element.from_tag('<text:page-number text:select-page="current"/>')
    assert isinstance(variable, VarPageNumber)


def test_var_page_number_from_tag2():
    variable = Element.from_tag('<text:page-number text:select-page="current"/>')
    assert variable.select_page == "current"


def test_var_page_count_class():
    variable = VarPageCount()
    assert isinstance(variable, VarPageCount)


def test_var_date_class():
    variable = VarDate()
    assert isinstance(variable, VarDate)


def test_var_date_from_tag():
    variable = Element.from_tag("<text:date></text:date>")
    assert isinstance(variable, VarDate)


def test_var_date_datetime():
    variable = VarDate(date=datetime(1977, 12, 25, 12, 15, 10), data_style="date_style")
    assert variable.date == "1977-12-25T12:15:10"


def test_var_date_style():
    variable = VarDate(date=datetime(1977, 12, 25, 12, 15, 10), data_style="date_style")
    assert variable.data_style == "date_style"


def test_var_date_adjust():
    variable = VarDate(
        date=datetime(1977, 12, 25, 12, 15, 10), date_adjust=timedelta(hours=2)
    )
    assert variable.date_adjust == "PT02H00M00S"


def test_var_time_class():
    variable = VarTime()
    assert isinstance(variable, VarTime)


def test_var_time_from_tag():
    variable = Element.from_tag(
        '<text:time text:time-value="1977-12-25T12:15:10">12:15:10</text:time>'
    )
    assert isinstance(variable, VarTime)


def test_var_time_style():
    variable = VarTime(data_style="date_style")
    assert variable.data_style == "date_style"


def test_var_time_adjust():
    variable = VarTime(time_adjust=timedelta(hours=2))
    assert variable.time_adjust == "PT02H00M00S"


def test_var_chapter_class():
    variable = VarChapter()
    assert isinstance(variable, VarChapter)


def test_var_chapter_from_tag():
    variable = Element.from_tag('<text:chapter text:display="name"/>')
    assert isinstance(variable, VarChapter)


def test_var_chapter_wrong_display():
    with pytest.raises(ValueError):
        VarChapter(display="wrong")


def test_var_file_name_class():
    variable = VarFileName()
    assert isinstance(variable, VarFileName)


def test_var_file_name_from_tag():
    variable = Element.from_tag(
        '<text:file-name text:display="name-and-extension">file.odt</text:file-name>'
    )
    assert isinstance(variable, VarFileName)


def test_var_file_name_from_tag2():
    variable = Element.from_tag(
        '<text:file-name text:display="name-and-extension">file.odt</text:file-name>'
    )
    assert variable.text == "file.odt"


def test_var_file_name_wrong_display():
    with pytest.raises(ValueError):
        VarFileName(display="wrong")


def test_var_initial_creator_class():
    variable = VarInitialCreator()
    assert isinstance(variable, VarInitialCreator)


def test_var_initial_creator_from_tag():
    variable = Element.from_tag(
        '<text:initial-creator text:fixed="true">John Doe</text:initial-creator>'
    )
    assert isinstance(variable, VarInitialCreator)


def test_var_initial_creator_from_tag2():
    variable = Element.from_tag(
        '<text:initial-creator text:fixed="true">John Doe</text:initial-creator>'
    )
    assert variable.text == "John Doe"


def test_var_initial_creator_fixed():
    variable = VarInitialCreator(fixed=True)
    assert variable.fixed is True


def test_var_creation_date_class():
    variable = VarCreationDate()
    assert isinstance(variable, VarCreationDate)


def test_var_creation_date_from_tag():
    variable = Element.from_tag("<text:creation-date/>")
    assert isinstance(variable, VarCreationDate)


def test_var_creation_date_fixed():
    variable = VarCreationDate(fixed=True)
    assert variable.fixed is True


def test_var_creation_date_data_style():
    variable = VarCreationDate(data_style="some_style")
    assert variable.data_style == "some_style"


def test_var_creation_time_class():
    variable = VarCreationTime()
    assert isinstance(variable, VarCreationTime)


def test_var_creation_time_from_tag():
    variable = Element.from_tag("<text:creation-time/>")
    assert isinstance(variable, VarCreationTime)


def test_var_creation_time_fixed():
    variable = VarCreationTime(fixed=True)
    assert variable.fixed is True


def test_var_creation_time_data_style():
    variable = VarCreationTime(data_style="some_style")
    assert variable.data_style == "some_style"


def test_var_description_class():
    variable = VarDescription()
    assert isinstance(variable, VarDescription)


def test_var_title_class():
    variable = VarTitle()
    assert isinstance(variable, VarTitle)


def test_var_subject_class():
    variable = VarSubject()
    assert isinstance(variable, VarSubject)


def test_var_keywords_class():
    variable = VarKeywords()
    assert isinstance(variable, VarKeywords)


def test_create_variable_set_float():
    variable_set = VarSet(ZOE, value=42)
    expected = (
        f'<text:variable-set text:name="{ZOE}" '
        'office:value-type="float" calcext:value-type="float" '
        'office:value="42" calcext:value="42" '
        'text:display="none"/>'
    )
    assert variable_set.serialize() == expected


def test_create_variable_set_datetime():
    date = datetime(2009, 5, 17, 23, 23, 00)
    variable_set = VarSet(ZOE, value=date, display=True)
    expected = (
        f'<text:variable-set text:name="{ZOE}" '
        'office:value-type="date" calcext:value-type="date" '
        'office:date-value="2009-05-17T23:23:00">'
        "2009-05-17T23:23:00"
        "</text:variable-set>"
    )
    assert variable_set.serialize() == expected


def test_create_variable_get():
    variable_get = VarGet(ZOE, value=42)
    expected = (
        f'<text:variable-get text:name="{ZOE}" '
        'office:value-type="float" calcext:value-type="float" '
        'office:value="42" calcext:value="42">'
        "42"
        "</text:variable-get>"
    )
    assert variable_get.serialize() == expected


def test_get_variable_decl(document):
    body = document.body
    name = "Variabilité"
    variable_decl = body.get_variable_decl(name)
    expected = f'<text:variable-decl office:value-type="float" text:name="{name}"/>'
    assert variable_decl.serialize() == expected


def test_get_variable_set(document):
    body = document.body
    name = "Variabilité"
    variable_sets = body.get_variable_sets(name)
    assert len(variable_sets) == 1
    expected = (
        f'<text:variable-set text:name="{name}" '
        'office:value-type="float" office:value="123" '
        'style:data-style-name="N1">123</text:variable-set>'
    )
    assert variable_sets[0].serialize() == expected


def test_get_variable_get(document):
    body = document.body
    value = body.get_variable_set_value("Variabilité")
    assert value == 123


# TODO On all the following variable tests, interact with the document


def test_create_page_number():
    page_number = VarPageNumber()
    expected = '<text:page-number text:select-page="current"/>'
    assert page_number.serialize() == expected


def test_create_page_number_complex():
    page_number = VarPageNumber(select_page="next", page_adjust=1)
    expected = '<text:page-number text:select-page="next" text:page-adjust="1"/>'
    assert page_number.serialize() == expected


def test_create_page_count():
    page_count = VarPageCount()
    expected = "<text:page-count/>"
    assert page_count.serialize() == expected


def test_create_date():
    date_element = VarDate(datetime(2009, 7, 20))
    expected = '<text:date text:date-value="2009-07-20T00:00:00">2009-07-20</text:date>'
    assert date_element.serialize() == expected


def test_create_date_fixed():
    date_element = VarDate(datetime(2009, 7, 20), fixed=True)
    expected = (
        '<text:date text:date-value="2009-07-20T00:00:00" '
        'text:fixed="true">'
        "2009-07-20"
        "</text:date>"
    )
    assert date_element.serialize() == expected


def test_create_date_text():
    date_element = VarDate(datetime(2009, 7, 20), text="20 juil. 09")
    expected = (
        '<text:date text:date-value="2009-07-20T00:00:00">20 juil. 09</text:date>'
    )
    assert date_element.serialize() == expected


def test_create_time_empty():
    time_element = VarTime()
    expected = '<text:time text:time-value="1900-01-01T00:00:00">00:00:00</text:time>'
    assert time_element.serialize() == expected


def test_create_time():
    time_element = VarTime(time(19, 30))
    expected = '<text:time text:time-value="1900-01-01T19:30:00">19:30:00</text:time>'
    assert time_element.serialize() == expected


def test_create_time_datetime():
    time_element = VarTime(datetime(2024, 2, 25, 19, 30, 15))
    expected = '<text:time text:time-value="2024-02-25T19:30:15">19:30:15</text:time>'
    assert time_element.serialize() == expected


def test_create_time_fixed():
    time_element = VarTime(time(19, 30), fixed=True)
    expected = (
        '<text:time text:time-value="1900-01-01T19:30:00" '
        'text:fixed="true">'
        "19:30:00"
        "</text:time>"
    )
    assert time_element.serialize() == expected


def test_create_time_text():
    time_element = VarTime(time(19, 30), text="19h30")
    expected = '<text:time text:time-value="1900-01-01T19:30:00">19h30</text:time>'
    assert time_element.serialize() == expected


def test_create_chapter():
    chapter = VarChapter()
    expected = '<text:chapter text:display="name"/>'
    assert chapter.serialize() == expected


def test_create_chapter_complex():
    chapter = VarChapter(display="number-and-name", outline_level=1)
    expected = '<text:chapter text:display="number-and-name" text:outline-level="1"/>'
    assert chapter.serialize() == expected


def test_create_filename():
    filename = VarFileName()
    expected = '<text:file-name text:display="full"/>'
    assert filename.serialize() == expected


def test_create_filename_fixed():
    filename = VarFileName(fixed=True)
    expected = '<text:file-name text:display="full" text:fixed="true"/>'
    assert filename.serialize() == expected


def test_create_initial_creator():
    element = VarInitialCreator()
    expected = "<text:initial-creator/>"
    assert element.serialize() == expected


def test_create_creation_date():
    element = VarCreationDate()
    expected = "<text:creation-date/>"
    assert element.serialize() == expected


def test_create_creation_time():
    element = VarCreationTime()
    expected = "<text:creation-time/>"
    assert element.serialize() == expected


def test_create_description():
    element = VarDescription()
    expected = "<text:description/>"
    assert element.serialize() == expected


def test_create_title():
    element = VarTitle()
    expected = "<text:title/>"
    assert element.serialize() == expected


def test_create_subject():
    element = VarSubject()
    expected = "<text:subject/>"
    assert element.serialize() == expected


def test_create_keywords():
    element = VarKeywords()
    expected = "<text:keywords/>"
    assert element.serialize() == expected



#!/usr/bin/env python
"""Minimal example of how to add a sublist to a list."""

import os
from pathlib import Path

from odfdo import Document, List, ListItem

_DOC_SEQUENCE = 27
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "minimal_list"
TARGET = "document.odt"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def generate_document() -> Document:
    """Return a text document with a list."""
    document = Document("text")
    body = document.body

    my_list = List(["chocolat", "café"])
    body.append(my_list)

    item = ListItem("thé")
    my_list.append(item)

    # A sublist is simply a list as an item of another list
    item.append(List(["thé vert", "thé rouge"]))

    print(body.serialize(True))

    return document


def main() -> None:
    document = generate_document()
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert len(document.body.get_lists()) == 2


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Create an empty text document and add a picture in a frame.
"""
from pathlib import Path

from odfdo import Document, Frame, Paragraph

_DOC_SEQUENCE = 65
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "basic_picture"
TARGET = "document.odt"
DATA = Path(__file__).parent / "data"
IMAGE = DATA / "newlogo.png"


def save_new(document: Document, name: str):
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def main():
    document = Document("text")
    body = document.body
    image_path = str(DATA / IMAGE)
    uri = document.add_file(image_path)
    image_frame = Frame.image_frame(
        uri,
        size=("6cm", "4cm"),
        position=("5cm", "10cm"),
    )

    # put image frame in a paragraph:
    paragraph = Paragraph("")
    paragraph.append(image_frame)
    body.append(paragraph)

    save_new(document, TARGET)


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
from __future__ import annotations

from datetime import date as dtdate
from datetime import datetime, timedelta
from decimal import Decimal

import pytest

from odfdo.element import Element
from odfdo.meta_user_defined import MetaUserDefined


def test_meta_user_defined_class():
    data = Element.from_tag("<meta:user-defined/>")
    assert isinstance(data, MetaUserDefined)


def test_meta_user_defined_create_no_param():
    with pytest.raises(ValueError):
        MetaUserDefined()


def test_meta_user_defined_create_no_value_type():
    with pytest.raises(ValueError):
        MetaUserDefined(name="key")


def test_meta_user_defined_create_no_value_string():
    data = MetaUserDefined(name="key", value_type="string")
    assert data.value == ""


def test_meta_user_defined_create_no_value_bool():
    data = MetaUserDefined(name="key", value_type="boolean")
    assert data.value is False


def test_meta_user_defined_create_no_value_date():
    with pytest.raises(AttributeError):
        MetaUserDefined(name="key", value_type="date")


def test_meta_user_defined_create_no_value_float():
    data = MetaUserDefined(name="key", value_type="float")
    assert data.value == 0


def test_meta_user_defined_create_no_value_time():
    with pytest.raises(TypeError):
        MetaUserDefined(name="key", value_type="time")


def test_meta_user_defined_create_string():
    data = MetaUserDefined(name="key", value_type="string", value="value")
    assert data.value == "value"


def test_meta_user_defined_create_bool():
    data = MetaUserDefined(name="key", value_type="boolean", value=True)
    assert data.value is True


def test_meta_user_defined_create_date():
    dt = dtdate(2025, 1, 2)
    data = MetaUserDefined(name="key", value_type="date", value=dt)
    assert data.value == datetime(2025, 1, 2)


def test_meta_user_defined_create_date_datetime():
    dt = datetime(2025, 1, 2)
    data = MetaUserDefined(name="key", value_type="date", value=dt)
    assert data.value == datetime(2025, 1, 2)


def test_meta_user_defined_create_date_timedelta():
    dt = timedelta(hours=3)
    data = MetaUserDefined(name="key", value_type="time", value=dt)
    assert data.value == dt


def test_meta_user_defined_create_float():
    data = MetaUserDefined(name="key", value_type="float", value=3.14)
    assert data.value == Decimal("3.14")


def test_meta_user_defined_create_int():
    data = MetaUserDefined(name="key", value_type="float", value=42)
    assert data.value == Decimal("42")


def test_meta_user_defined_from_xml():
    xml = (
        "<meta:user-defined "
        'meta:name="Référence" meta:value-type="boolean">'
        "true"
        "</meta:user-defined>"
    )
    data = Element.from_tag(xml)
    assert data.name == "Référence"
    assert data.value_type == "boolean"
    assert data.value is True



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
"""Header class for "text:h" tag."""

from __future__ import annotations

from re import sub
from typing import Any

from .element import PropDef, register_element_class
from .mixin_md import MDHeader
from .paragraph import Paragraph


class Header(Paragraph, MDHeader):
    """A title, a specialized paragraph, "text:h".

    Attributes:
        level (int): The outline level of the header. Level count begins at 1.
        text (str or None): The content of the header.
        restart_numbering (bool): If True, numbering restarts at this header level.
        start_value (int or None): The value at which to start numbering.
        suppress_numbering (bool): If True, no numbering for this header.
        style (str or None): The style name of the header.
    """

    _tag = "text:h"
    _properties = (
        PropDef("level", "text:outline-level"),
        PropDef("restart_numbering", "text:restart-numbering"),
        PropDef("start_value", "text:start-value"),
        PropDef("suppress_numbering", "text:suppress-numbering"),
    )

    def __init__(
        self,
        level: int = 1,
        text: str | None = None,
        restart_numbering: bool = False,
        start_value: int | None = None,
        suppress_numbering: bool = False,
        style: str | None = None,
        formatted: bool = True,
        **kwargs: Any,
    ) -> None:
        """Create a header element "text:h" of the given style and level, containing the
        optional given text.

        If "formatted" is True (the default), the given text is appended with <CR>,
        <TAB> and multiple spaces replaced by ODF corresponding tags.

        Args:
            level: The outline level of the header (starts at 1).
            text: The initial text content of the header.
            restart_numbering: If True, restart numbering at this level.
            start_value: The value at which to start numbering.
            suppress_numbering: If True, suppresses numbering for this header.
            style: The style name for the header.
            formatted: If True, replace special characters in `text` with ODF tags.
        """
        super().__init__(**kwargs)
        if self._do_init:
            self.level = int(level)
            if text:
                if formatted:
                    self.text = ""
                    self.append_plain_text(text)
                else:
                    self.text = self._unformatted(text)
            if restart_numbering:
                self.restart_numbering = True
            if start_value is not None:
                self.start_value = start_value
            if suppress_numbering:
                self.suppress_numbering = True
            # if style:
            #     self.style = style

    def get_formatted_text(
        self,
        context: dict | None = None,
        simple: bool = False,
    ) -> str:
        if not context:
            context = {
                "document": None,
                "footnotes": [],
                "endnotes": [],
                "annotations": [],
                "rst_mode": False,
                "img_counter": 0,
                "images": [],
                "no_img_level": 0,
            }
        context["no_img_level"] += 1
        title = super().get_formatted_text(context)
        context["no_img_level"] -= 1
        title = title.strip()
        title = sub(r"\s+", " ", title)

        # No rst_mode ?
        if not context["rst_mode"]:
            return title
        # If here in rst_mode!

        # Get the level, max 5!
        LEVEL_STYLES = "#=-~`+^°'."
        level = int(self.level)
        if level > len(LEVEL_STYLES):
            raise ValueError("Too many levels of heading")

        # And return the result
        result = ["\n", title, "\n", LEVEL_STYLES[level - 1] * len(title), "\n"]
        return "".join(result)


Header._define_attribut_property()

register_element_class(Header)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""RubyBase class for "text:ruby-base" tag."""

from __future__ import annotations

from .annotation import AnnotationMixin
from .bookmark import BookmarkMixin
from .element import register_element_class
from .mixin_link import LinkMixin
from .note import NoteMixin
from .reference import ReferenceMixin
from .user_field import UserDefinedMixin


class RubyBase(
    UserDefinedMixin,
    LinkMixin,
    ReferenceMixin,
    BookmarkMixin,
    AnnotationMixin,
    NoteMixin,
):
    """Represents the base text for a ruby annotation, "text:ruby-base".

    This element contains the text content that is to be annotated. It can
    include any paragraph element content, such as text spans. The
    `text:style-name` attribute of this element references a ruby style
    that defines the formatting attributes for the ruby annotation.

    Inherits from:
        - :class:`~odfdo.annotation.AnnotationMixin`
        - :class:`~odfdo.bookmark.BookmarkMixin`
        - :class:`~odfdo.note.NoteMixin`
        - :class:`~odfdo.reference.ReferenceMixin`
    """

    _tag = "text:ruby-base"


register_element_class(RubyBase)



#!/usr/bin/env python
"""Example of methods and properties to analyse a document.

These methods return a single element (or None):

    - `body.get_note(position)`
    - `body.get_paragraph(position)`
    - `body.get_header(position)`
"""

# Expected result on stdout:
# - Content of the first footnote:
# 1. [Gaiman, Neil](http://en.wikipedia.org/w/index.php?title=Neil_Gaiman)
# (2003). Don't Panic: Douglas Adams and the "Hitchhiker's Guide to the
# Galaxy". Titan Books. pp 144-145. ISBN 1-84023-742-2.
#
# - Content of the paragraph with the word 'Fish'
# In So Long, and Thanks for All the Fish (published in 1984), Arthur
# returns home to Earth, rather surprisingly since it was destroyed when
# he left. He meets and falls in love with a girl named
# [Fenchurch](http://en.wikipedia.org/w/index.php?title=Minor_characters_from_The_Hitchhiker%27s_Guide_to_the_Galaxy%23Fenchurch), and discovers this Earth is a replacement provided by the [dolphin](http://en.wikipedia.org/w/index.php?title=Dolphin)s in their Save the Humans campaign. Eventually he rejoins Ford, who claims to have saved the Universe in the meantime, to hitch-hike one last time and see God's Final Message to His Creation. Along the way, they are joined by Marvin, the Paranoid Android, who, although 37 times older than the universe itself (what with time
# travel and all), has just enough power left in his failing body to read
# the message and feel better about it all before expiring.
#
# - Content of the first Title:
# The Hitchhiker's Guide to the Galaxy
#
# - Content of the last Title:
# Official sites

import os
import sys
from pathlib import Path

from odfdo import Document

_DOC_SEQUENCE = 70
DATA = Path(__file__).parent / "data"
SOURCE = "collection2.odt"


def read_source_document() -> Document:
    """Return the source Document."""
    try:
        source = sys.argv[1]
    except IndexError:
        source = DATA / SOURCE
    return Document(source)


def method_demo(document: Document) -> None:
    """Show some methos examples."""
    # The body object is an XML element from which we can access one or several
    # other elements we are looking for.
    body = document.body

    # Accessing a single element
    # To access a single element by name, position or a regular expression on
    # the content, use get_xxx_by_<criteria>, where criteria can be position,
    # content, or for some of them name, id title, description.
    print("- Content of the first footnote:")
    print(str(body.get_note(position=0)))
    print()
    print("- Content of the paragraph with the word 'Fish'")
    print(str(body.get_paragraph(content="Fish")))
    print("- Content of the first Title:")
    print(str(body.get_header(position=0)))
    print("- Content of the last Title:")
    print(str(body.get_header(position=-1)))


def main() -> None:
    document = read_source_document()
    method_demo(document)
    test_unit(document)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    body = document.body
    assert str(body.get_note(position=0)).startswith(
        "1. [Gaiman, Neil](http://en.wikipedia.org/w/index.php?title=Neil_Gaiman) (2003)"
    )
    assert str(body.get_paragraph(content="Fish")).endswith("all before expiring.\n")
    assert str(body.get_header(position=0)).startswith("The Hitchhiker's Guide")
    assert str(body.get_header(position=-1)).startswith("Official sites")


if __name__ == "__main__":
    main()
